/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/* 
 * File:   FixedAnimationContainer.cpp
 * Author: Martin Kjellqvist <martin.kjellqvist@miun.se>
 * 
 * Created on March 14, 2018, 8:46 PM
 */

#include "FixedAnimationContainer.h"
#include "Tile.h"
#include "Tween.h"
#include <algorithm>

FixedAnimationContainer::FixedAnimationContainer(int grid_rows, int grid_columns) :
grid_columns(grid_columns), grid_rows(grid_rows) {
    enable_mouse_move(true);
    add_events(Gdk::POINTER_MOTION_MASK);

    set_size_request(grid_columns * col_grid_width(), grid_rows * row_grid_width());

    auto detector_callback = [this](gdouble x, gdouble y, MotionDetector::Direction d, double speed) {
        this->on_mouse_motion(x, y, d, speed);
    };
    detector.callback = detector_callback;
}

void FixedAnimationContainer::put_in_grid(Gtk::Widget& widget, int row, int col, int size_x, int size_y) {
    auto grid_size_x = col_grid_width();
    auto grid_size_y = row_grid_width();
    auto x = col * grid_size_x;
    auto y = row * grid_size_y;

    // Already in grid? Just set the position
    if(widget.get_parent() != nullptr){
        child_property_x(widget).set_value(x);
        child_property_y(widget).set_value(y);
        return;
    }
    widget.set_size_request(grid_size_x * size_x, grid_size_y * size_y);
    widget.show();
    put(widget, x, y);
}

bool inside(int value, int start, int size) {
    return value >= start && value < start + size;
}

Gtk::Widget* FixedAnimationContainer::widget_at(int x, int y) {
    auto children = get_children();

    auto contains_xy = [x, y, this](Gtk::Widget * widget) {
        auto width = widget->get_width();
        auto height = widget->get_height();
        auto c_x = this->child_property_x(*widget).get_value();
        auto c_y = this->child_property_y(*widget).get_value();
        if (inside(x, c_x, width) && inside(y, c_y, height))
            return true;
        return false;
    };
    auto widget = std::find_if(children.begin(), children.end(), contains_xy);
    if (widget != children.end())
        return *widget;
    return nullptr;
}

void FixedAnimationContainer::enable_mouse_move(bool enable) {
    mouse_moves_widgets = enable;
}

void FixedAnimationContainer::on_mouse_motion(gdouble x, gdouble y, MotionDetector::Direction d, double speed) {
    if (mouse_moves_widgets) {
        if(auto widget = widget_at(x, y)){
            move(widget, d, speed);
        }
    }
}

void FixedAnimationContainer::move(Gtk::Widget* widget, MotionDetector::Direction d, double speed) {
    auto property_x = this->child_property_x(*widget);
    auto property_y = this->child_property_y(*widget);
    auto grid_size_x = col_grid_width(); //get_width() / grid_columns;
    auto grid_size_y = row_grid_width(); // get_height() / grid_rows;
    auto on_complete_calllback = [this](){
        this->on_animation_complete();
    };
    switch (d) {
        case MotionDetector::RIGHT:
            make_tween(grid_size_x, col_grid_width() / speed, property_x, BOUNCE, on_complete_calllback)();
            break;
        case MotionDetector::LEFT:
            make_tween(-grid_size_x, col_grid_width() / speed, property_x, BOUNCE, on_complete_calllback)();
            break;
        case MotionDetector::DOWN:
            make_tween(grid_size_y, row_grid_width() / speed, property_y, BOUNCE, on_complete_calllback)();
            break;
        case MotionDetector::UP:
            make_tween(-grid_size_y, row_grid_width() / speed, property_y, BOUNCE, on_complete_calllback)();
            break;
    }
}

void FixedAnimationContainer::connect_detector(Gtk::Widget* widget) {
    widget->signal_motion_notify_event().connect(detector, true);
}

int FixedAnimationContainer::col_grid_width() const {
    return 100;
}

int FixedAnimationContainer::row_grid_width() const {
    return 100;
}

void FixedAnimationContainer::on_animation_complete() {

}
