/* 
 * File:   FixedAnimationContainer.h
 * Author: Martin Kjellqvist <martin.kjellqvist@miun.se>
 *
 * Created on March 14, 2018, 8:46 PM
 */

#ifndef FIXEDANIMATIONCONTAINER_H
#define FIXEDANIMATIONCONTAINER_H

#include <gtkmm/fixed.h>
#include <gtkmm/widget.h>
#include "MotionDetector.h"

class FixedAnimationContainer : public Gtk::Fixed {
public:
    
    /**
     * Enable_mouse_moves is on by default.
     * @param grid_rows
     * @param grid_columns
     */
    FixedAnimationContainer(int grid_rows = 5, int grid_columns = 4);

    /**
     * Inserts a widget in to the grid, resizing the widget according to the grid.
     * Use this after the container is displayed. 
     * Resizing the container will produce inconsistent results.
     * @param widget
     * @param row
     * @param col
     * @param rowspan number of cells wide
     * @param colspan number of cells high
     */
    void put_in_grid(Gtk::Widget& widget, int row, int col, int rowspan = 1, int colspan = 1);

    /**
     * Finds a widget in container coordinates.
     * @param x Container x coordinate
     * @param y Container y coordinate
     * @return the widget in the current position or nullptr if no widget is under current position.
     */
    Gtk::Widget * widget_at(int x, int y);
    
    void enable_mouse_move(bool enable);
    void connect_detector(Gtk::Widget* widget);
    
    virtual void move(Gtk::Widget* widget, MotionDetector::Direction d, double speed);

    int row_grid_width() const;
    int col_grid_width() const;
    
protected:
    
    void on_mouse_motion(gdouble x, gdouble y, MotionDetector::Direction d, double speed);
    virtual void on_animation_complete();
    int grid_columns;
    int grid_rows;
    bool mouse_moves_widgets;
    MotionDetector detector;
};

#endif /* FIXEDANIMATIONCONTAINER_H */

