/* 
 * File:   GinkoState.cpp
 * Author: Martin Kjellqvist <martin.kjellqvist@miun.se>
 * 
 * Created on February 25, 2018, 5:20 PM
 */

#include "GinkoState.h"
#include <iostream>

// Huffman codes
const std::string C_SQUARE = "0";
const std::string C_VERTICAL = "10";
const std::string C_HORIZONTAL = "1111";
const std::string C_BIG = "1110";
const std::string C_OPEN = "110";
/**
 *           ROOT
 *        0 /    \ 1
 *     SQ(4)    0/ \ 1
 *             V   /\
 *                O  /\
 *                  B  H
 */
// S4, V4, O2, H1, B1
// BH 2
// OBH 4
// OBHV 8
// OBHVS 12

std::string state_for_tile(TileType tiletype) {

    switch (tiletype) {
        case SQUARE:
            return C_SQUARE;
        case VERTICAL:
            return C_VERTICAL;
        case HORIZONTAL:
            return C_HORIZONTAL;
        case BIG:
            return C_BIG;
        case EMPTY:
            return C_OPEN;
        default:
            return "";
    }
}
#include <cassert>

std::vector<TileType> tiles_for_state(board_state_t state) {
    std::string state_string = state;

    std::vector<TileType> result;

    auto pop_front = [](std::string & s) {
        char c = s.front();
        s.erase(s.begin());
        return c;
    };
    // The string is easily optimized to char compares.
    // It is written like this because the names are nicer in the if 
    // statements.
    while (!state_string.empty()) {
        std::string match;
        match += pop_front(state_string);
        if (match == C_SQUARE) {
            result.push_back(SQUARE);
            continue;
        }
        match += pop_front(state_string);
        if (match == C_VERTICAL) {
            result.push_back(VERTICAL);
            continue;
        }
        match += pop_front(state_string);
        if (match == C_OPEN) {
            result.push_back(EMPTY);
            continue;
        }
        match += pop_front(state_string);
        if (match == C_BIG){
            result.push_back(BIG);
            continue;
        }
        assert(match == C_HORIZONTAL);
        result.push_back(HORIZONTAL);
    }
    return result;

}