/* 
 * File:   Tile.h
 * Author: martin
 *
 * Created on February 12, 2018, 2:04 PM
 */

#ifndef TILE_H
#define TILE_H

#include <gtkmm.h>
#include "GinkoState.h"
#include <algorithm>
#include <numeric>

struct null_parent:std::runtime_error{
    null_parent(const std::string& __arg);
};

static const std::string tile_css = "* { color: #ff00ea; background-color: #040404; padding:10px; border: 10px solid #FFFFFF; } ";


class Tile: public Gtk::Image {
public:
    Tile(TileType tiletype);
    Gtk::ChildPropertyProxy<int> property_x();
    Gtk::ChildPropertyProxy<int> property_y();
    Gtk::Fixed* get_parent();
    TileType get_tile_type() const;
    int get_row_span() const;
    int get_col_span() const;
private:
    TileType type;
};

#include <algorithm>

/**
 * @ttype Iterator to Tile objects.
 */
template<typename T>
board_state_t accumulated_state(T begin, T end){
    auto add_tile_states = [](const std::string& lhs, const Tile& rhs)->std::string{
        return lhs + state_for_tile(rhs.get_tile_type());
    };
    auto text_state = std::accumulate(begin, end, std::string(), add_tile_states);
    return std::stoi(text_state, nullptr, 2);
}

#endif /* TILE_H */
