/* 
 * File:   TileBoard.h
 * Author: martin
 *
 * Created on February 14, 2018, 10:37 AM
 */

#ifndef TILEBOARD_H
#define TILEBOARD_H

#include <gtkmm.h>
#include <vector>
#include <utility>
#include "Tween.h"
#include "Tile.h"
#include "FixedAnimationContainer.h"
#include "GinkoSolver.h"

MotionDetector::Direction get_direction(Move m);

class TileBoard : public FixedAnimationContainer {
public:
    enum Setup {
        DEFAULT,  // Blocked by Broad Sword and Standing Horses
        VARIANT1, // Can’t Fly Out, Even with Wings
        VARIANT2, // Layered Blockage
        VARIANT3, // Blocked on the East
        VARIANT4, // Exit Blocked
        VARIANT5  // Totally Blocked
    };

    using on_win_callback = std::function<void()>;
    using on_move_callback = std::function<void()>;
    
    on_win_callback on_win;
    on_move_callback on_move;
    
    TileBoard(Setup config = DEFAULT);

    void setup_tiles();
    
    void set_images(std::map<TileType, Glib::RefPtr<Gdk::Pixbuf>> images);
    /**
     * Changes board state to reflect the move if the move is possible.
     * Animates the move. See set_tween_type.
     * 
     * row, col does not have to be the top left corner.
     * 
     * @param row col from
     * @return true move is possible, the board state has changed.
     */
    bool move(int row, int col, MotionDetector::Direction to, double speed = 1);

    /**
     * Relays to move(row, col, d, speed).
     * @param widget
     * @param d
     * @param speed
     */
    virtual void move(Gtk::Widget* widget, MotionDetector::Direction d, double speed) override;
    
    /**
     * Unique representation of the current board state.
     * @return state.
     */
    board_state_t state();
    /**
     * 
     * @return 
     */

    std::pair<char, char> get_tile_row_col(Tile * tile);
    
    /**
     * Finds a tile if the top left corner of the tile is at the given position.
     * @param row
     * @param col
     * @return nullptr if no tile has a top left corner at row,col.
     */    
    Tile* get_tile_at(int row, int col);
    
    /**
     * Clumsy, checks to see if a tile is occupied by any non empty tile.
     * 
     * @param row
     * @param col
     * @return 
     */
    bool is_occupied(int row, int col);
    
    void set_state(board_state_t state);
    
    bool is_winning();
    
    solution_t get_solution();
    
    void on_animation_complete() override;

    void set_interactive(bool interactive);
    
    bool is_interactive() const;
    
private:
    std::map<TileType, Glib::RefPtr<Gdk::Pixbuf> > images;
    void fix_empty_tiles();
    void insert_tile(Tile* tile, int row, int col);
    std::vector<Tile*> board;
    bool interactive;
    std::unique_ptr<GinkoSolver> solver;
};

#endif /* TILEBOARD_H */
